import inspect
import os
from functools import partial
from unittest import mock

import mrbenn_panel
import pytest
from django.contrib.sites.models import Site
from django.db import transaction
from django.test import TestCase
from django.test import TestCase, override_settings
from django.test.client import RequestFactory
from django.urls import reverse
from mrbenn_panel import views
from mrbenn_panel.views import _retrieve_view_name, _retrieve_template_name

set_env_vars = partial(mock.patch.dict, os.environ)


class DjangoReadOnlyTests(TestCase):

    def setUp(self):
        self.rf = RequestFactory()
        # below could be retrieved from test.client but for time being hand-coding
        self.testserver = 'testserver'

    def test_retrieve_view_name(self):
        referrer_url = self.testserver + reverse('djdt:open_view')

        my_request = self.rf.get('', HTTP_REFERER=referrer_url)
        view_name = _retrieve_view_name(my_request)

        self.assertEqual(type(view_name), str)
        self.assertEqual(view_name, inspect.getsourcefile(views))

    def test_retrieve_template_name(self):
        referrer_url = self.testserver + reverse('djdt:open_template')

        some_site_url = ''

        my_request = self.rf.get(some_site_url, {'template': 'index.html'}, HTTP_REFERER=referrer_url)
        template_name = _retrieve_template_name(my_request)

        self.assertEqual(type(template_name), str)
        template_name = template_name.replace("\\", "/") # potential issue over operating systems elsewise
        self.assertTrue(template_name.endswith('templates/index.html'))



# -- Project information -----------------------------------------------------
import datetime

project = "mrbenn-toolbar-plugin"
copyright = "{}, Andy Woods and contributors"
copyright = copyright.format(datetime.date.today().year)
author = "Andy Woods and contributors"

version = "0.0.2"

html_theme = "default"

master_doc = 'index'

extensions = [
    'sphinx.ext.duration',
    'sphinx.ext.doctest',
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.intersphinx',
]


import os
import subprocess
import sys

from django.conf import settings
from django.http import JsonResponse
from django.template.loader import get_template
from django.urls import resolve


def _retrieve_template_name(request):
    template_name = request.GET.get('template')
    template = get_template(template_name)
    return template.origin.name


def open_template(request):
    template_name = _retrieve_template_name(request)
    return open_element(template_name)


def _extract_func(request):
    host = request.get_host()
    referrer = request.META['HTTP_REFERER']
    url_ending = referrer.split(host)[1]
    match = resolve(url_ending)
    func, _, _ = match
    return func


def _retrieve_view_name(request):
    func = _extract_func(request)
    # was having issues with parent classes being returned not class itself so using below (and not inspect)
    # https://stackoverflow.com/questions/697320/how-do-i-get-the-filepath-for-a-class-in-python
    el = os.path.abspath(sys.modules[func.__module__].__file__)
    return el


def open_view(request):
    view_name = _retrieve_view_name(request)
    return open_element(view_name)


class MustSpecifyIDE(BaseException):
    pass


class ProblemWithIDEPath(BaseException):
    pass


def open_element(el):
    if sys.platform == 'win32':
        ide = None
        try:
            ide = settings.DJANGO_WINDOWS_IDE
        except AttributeError:
            pass
        if not ide:
            raise MustSpecifyIDE(f'To open {el} with your default IDE please specify '
                                 f'DJANGO_WINDOWS_IDE path in your settings')
        try:
            subprocess.check_call([ide, el])
        except FileNotFoundError:
            raise ProblemWithIDEPath(f'The path your specified for your IDE executable (via DJANGO_WINDOWS_IDE in '
                                     f'settings) is incorrect: {ide}')


    elif sys.platform == 'darwin':
        subprocess.call(('open', el))

    else:
        try:
            subprocess.Popen(['xdg-open', el])
        except OSError:
            pass
    return JsonResponse({})



from debug_toolbar.panels import Panel
# Code taken and adapted from Simon Willison and Django Snippets:
# https://www.djangosnippets.org/snippets/766/
from debug_toolbar.panels.templates import TemplatesPanel
from debug_toolbar.utils import get_name_from_obj
from django.template import loader
from django.test.signals import template_rendered
from django.urls import path, resolve
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from mrbenn_panel import views


class MrBennPanel(TemplatesPanel):
    """
    A panel that lists all templates used during processing of a response.
    """
    template_names = []

    nav_title = _("Load in IDE")

    @property
    def title(self):
        return 'From panel'

    @property
    def nav_subtitle(self):
        """
        Show abbreviated name of view function as subtitle
        """

        match = resolve(self.toolbar.request.path)
        func, args, kwargs = match
        view_name = get_name_from_obj(func)

        template = loader.get_template('mrbenn_panel/mrbenn_title.html')
        context = {
            'template_name': self.templates[0]["template"].name if self.templates else '',
            'view_name': view_name,
        }
        return mark_safe(template.render(context=context))

    template = "mrbenn_panel/mrbenn_panel.html"

    @classmethod
    def get_urls(cls):
        return [
            path("open_template/", views.open_template, name="open_template"),
            path("open_view/", views.open_view, name="open_view"),
        ]

    def generate_stats(self, request, response):
        pass




import os
from typing import List

import django

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

SECRET_KEY = "NOTASECRET"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
        "ATOMIC_REQUESTS": True,
    },
    "other": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
        "ATOMIC_REQUESTS": True,
    },
}

ALLOWED_HOSTS: List[str] = []

INSTALLED_APPS = [
    "django.contrib.sites",
    "debug_toolbar",
    "mrbenn_panel",
]

MIDDLEWARE: List[str] = ["debug_toolbar.middleware.DebugToolbarMiddleware", ]

ROOT_URLCONF = "tests.urls"
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True

if django.VERSION < (4, 0):
    USE_L10N = True

SITE_ID = 1

USE_TZ = True

DEBUG_TOOLBAR_PANELS = [
    "mrbenn_panel.panel.MrBennPanel",
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [BASE_DIR + '/tests/templates'],
        'APP_DIRS': True,
    },
]


from django.urls import include, path
from django.views.generic import TemplateView
import debug_toolbar

urlpatterns = [
    path("__debug__/", include(debug_toolbar.urls)),
    path("", TemplateView.as_view(template_name='index.html'))
]


import pathlib
from setuptools import setup


HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()
setup(
    name="mrbenn-toolbar-plugin",
    version="0.0.2",
    description="An addon for Django Debug Toolbar that allows for directly opening static files and views",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/andytwoods/mrbenn",
    author="Andy Woods",
    author_email="andytwoods@gmail.com",
)



import inspect

from django.test import TestCase
from django.urls import reverse
from django.test.client import RequestFactory

from mrbenn_panel import views
from mrbenn_panel.views import _retrieve_view_name, _retrieve_template_name


class TestLoadInIDE(TestCase):

    def setUp(self):
        self.rf = RequestFactory()
        # below could be retrieved from test.client but for time being hand-coding
        self.testserver = 'testserver'


    def test_retrieve_view_name(self):
        referrer_url = self.testserver + reverse('djdt:open_view')

        my_request = self.rf.get('', HTTP_REFERER=referrer_url)
        view_name = _retrieve_view_name(my_request)

        self.assertEquals(type(view_name), str)
        self.assertEquals(view_name, inspect.getsourcefile(views))

    def test_retrieve_template_name(self):
        referrer_url = self.testserver + reverse('djdt:open_template')

        my_request = self.rf.get('', {'template': 'index.html'}, HTTP_REFERER=referrer_url)
        template_name = _retrieve_template_name(my_request)

        self.assertEquals(type(template_name), str)
        self.assertTrue('example/templates/index.html' in template_name)
